@@ -27,6 +27,74 @@ from sklearn.metrics import f1_score, classification_report, confusion_matrix
 
 
 
+from counter import Counter
+from nans import Nans
+
+
+class Snaplib__222:
+    '''
+    data preprocessing library
+    '''
+    def __init__(self, start=0):
+        self.counter_cross_val = Counter(start)
+
+
+
+
+    @staticmethod
+    def nan_info(df:pd.DataFrame) -> pd.DataFrame:
+        '''
+        Get pandas.DataFrame with information about missing data in columns
+        Use case:
+        missing_info_df = Snaplib().nan_info(df)
+        '''
+
+        if not isinstance(df, pd.core.frame.DataFrame):
+            raise TypeError('The df must be a pandas.core.frame.DataFrame instance.')
+        return Nans().nan_info(df)
+
+
+
+
+    @staticmethod
+    def nan_plot(df:pd.DataFrame) -> plt.figure:
+        '''
+        Visualise missing data in pandas.DataFrame
+        Use case:
+        Snaplib().nan_plot(df)
+        '''
+
+        if not isinstance(df, pd.core.frame.DataFrame):
+            raise TypeError('The df must be a pandas.core.frame.DataFrame instance.')
+        return Nans().nan_plot(df)
+
+
+
+    
+    @staticmethod
+    def cleane(df:pd.DataFrame, 
+                target:str = None, 
+                verbose:bool = True) -> pd.DataFrame:
+        '''
+        drop_duplicates
+        drop rows with nan in target
+        drop column with 1 unique value
+        Use case:
+        df = Snaplib().cleane(df, target_name, verbose=True)
+        '''
+
+        if not isinstance(df, pd.core.frame.DataFrame):
+            raise TypeError('The df must be a pandas.core.frame.DataFrame instance.')
+        if target is not None:
+            if type(target) != str:
+                raise TypeError('target must be of str type.')
+        if type(verbose) != int and type(verbose) != bool:
+            raise TypeError('verbose must be of int type or bool.')
+        return Nans().cleane(df, target=None, verbose=True)
+
+
+
+
 
 
 
@@ -94,7 +162,6 @@ class Snaplib:
     def nan_info(self, df):
         '''
         Get pandas.DataFrame with information about missing data in columns
-
         Use case:
         missing_info_df = Snaplib().nan_info(df)
         '''
@@ -123,7 +190,6 @@ class Snaplib:
     def nan_plot(self, df):
         '''
         Visualise missing data in pandas.DataFrame
-
         Use case:
         Snaplib().nan_plot(df)
         '''
@@ -155,7 +221,6 @@ class Snaplib:
 
         Use case:
         df = Snaplib().cleane(df, target_name, verbose=True)
-        
         '''
 
         if not isinstance(df, pd.core.frame.DataFrame):
@@ -227,7 +292,6 @@ class Snaplib:
     def encode_column(self, df, column):
         '''
         encode one column in dataframe
-
         Use case:
         df = Snaplib().encode_column(df, column_name_str)
         '''
@@ -1436,7 +1500,7 @@ class Snaplib:
 
 
 
-    def recover_data(self,
+    def recover_data(self, 
                      df_0, 
                      device='cpu',
                      verbose = 1,
@@ -1460,30 +1524,30 @@ class Snaplib:
             raise TypeError('verbose must be of int type or bool.')
         if not isinstance(discrete_columns, list) and discrete_columns != 'auto':
             raise TypeError('The discrete_columns must be a list instance or string "auto".')
-        
+
 
 
         counter_predicted_values = 0
-        CLASS_VALUE_COUNTS = 30
+        CLASS_VALUE_COUNTS = 20
         K_FOLDS = 4
-        
-        
+
+
         warning_switch = 'WARNING:\nYou have a lot of unique values for this discrete column, more tahn 100.\
                                     \nPrediction and imputing has been switched to regression. \
                                     \nTry encode this column and restart this algorithm.\
                                     \nOtherwise you can include this feature to argument discrete_columns=[] \
                                     \nfor forced classification. It will take a very long time.\n'
 
-        warning_time = 'WARNING:\nYou have a lot of unique values for this column and classification, more tahn 30.\
+        warning_time = 'WARNING:\nYou have a lot of unique values for this column and classification, more tahn 50.\
                                 \nIt will take a very long time. Probably encoding and restart improve the situation.'
 
 
         advice_text = 'ADVICE:  \nYou can try include this feature to argument discrete_columns=[] \
                                 \nfor forced classification. Probably it improve precision. \
                                 \nOr set discrete_columns="auto"'
-        
-        
-        
+
+
+
 
 
         def get_predictors(columns, target_column):
@@ -1491,7 +1555,16 @@ class Snaplib:
             if target_column in columns:
                 columns_now.remove(target_column)
             return columns_now
-
+        
+        
+        def chek_int(ls):
+            set_ls = list(set([el.split('.')[1] for el in ls]))
+            if len(set_ls) == 1 and set_ls[0] == '0':
+                condition = True
+            else:
+                condition = False
+            return condition
+        
 
         def normalize_data(df_in, columns):
             df = df_in.copy()
@@ -1502,12 +1575,12 @@ class Snaplib:
                 df[col] = np.log1p(df[col])
             return df
 
-        
+
         def denormalize_targ(arr, min_y, max_y):
             arr = np.expm1(arr)
             arr = (arr * (max_y - min_y)) + min_y
             return arr
-            
+
 
 
 
@@ -1562,50 +1635,52 @@ class Snaplib:
             miss_df = df_normzd.loc[miss_indeces, predictors]
             # X data for train and model evaluation 
             work_df = df_normzd.iloc[work_indeces, : ]
-            
-            
+
+
             target_values_counted = work_df[target_now].value_counts()
             len_target_values_counted = len(target_values_counted)
             feature_type_target = data_info.loc[target_now, 'col_type']
-                        
-            
+
+
             # Info
             if verbose:
                 percent_missing_data = data_info.loc[target_now, 'NaN_percent']
                 print(f'Feature: {target_now}, missing values: {percent_missing_data}%\n')
-            
+
 
             # FORK to classification or regression
             classification = False
             to_int_flag = False
-            
+
             if feature_type_target == 'object':
                 if len_target_values_counted > 100:
                     classification = False
                     print(colored(target_now, 'red'))
                     print(colored(warning_switch, 'red'))
                     to_int_flag = True
-                    
+
             if isinstance(discrete_columns, list):
                 if target_now in discrete_columns:
                     classification = True
-                    if len_target_values_counted > CLASS_VALUE_COUNTS:
+                    if len_target_values_counted > 50:
                         if verbose:
                             print(colored(warning_time, 'yellow'))
-                        
+
                 elif len_target_values_counted <= CLASS_VALUE_COUNTS:
                     if verbose:
                         print(colored(advice_text, 'green'))
-                        
+
             if discrete_columns == 'auto':
-                if len_target_values_counted <= CLASS_VALUE_COUNTS:
-                    if len_target_values_counted <= 10:
-                        classification = True
-                    elif int(target_values_counted.index.min()) == 0 and target_values_counted.index.max() == len_target_values_counted -1 and len_target_values_counted < 100:
-                        classification = True
-                    elif int(target_values_counted.index.min()) == 1 and target_values_counted.index.max() == len_target_values_counted  and len_target_values_counted < 100:
-                        classification = True
-                    
+                str_floats = np.array(work_df[target_now].value_counts(dropna=False).index).astype(str).tolist()
+                ints = chek_int(str_floats)
+                if len_target_values_counted <= CLASS_VALUE_COUNTS and ints:
+                    classification = True
+
+                        
+            # print(target_values_counted)
+            # print(len_target_values_counted)
+            # print('classification', classification)
+            # print('to_int_flag', to_int_flag)
 
             if classification:
                 # Test
@@ -1613,7 +1688,7 @@ class Snaplib:
                     print('CLASSIFIER cross validation:')                    
                     test_y_all = np.array([])
                     pred_all = np.array([])
-                    
+
                     k_fold_dict = self.k_folds_split(work_df, target_now, K_FOLDS)
                     for k in range(0, K_FOLDS):
                         lgb_class = lgb.LGBMClassifier(random_state=0, n_jobs=-1, device=device)
@@ -1637,26 +1712,28 @@ class Snaplib:
                     print(f'random 15 y_test: {test[:15]}')
                     print(f'random 15 y_pred: {pred[:15]}\n')
 
-
-
                 X = work_df[predictors]
                 y = work_df[target_now]    
-                
+
                 # Final prediction & imputing
                 lgb_class = lgb.LGBMClassifier(random_state=0, n_jobs=-1, device=device)
                 lgb_class.fit(X, y)
                 pred_miss = lgb_class.predict(miss_df)
-                
+
                 if verbose:
-                    print(f'first 10 imputed: {np.round(pred_miss[:10], 1)}\n')
+                    print(f'first 15 imputed: {np.round(pred_miss[:15], 1)}\n')
 
                 df.loc[miss_indeces, target_now] = np.array(pred_miss)
                 counter_predicted_values += len(miss_indeces)
 
 
-            
+
             # regression for target_now 
             else:
+                # str_floats = np.array(work_df[target_now].value_counts(dropna=False).index).astype(str).tolist()
+                # to_int_flag = chek_int(str_floats)
+                # print('to_int_flag', to_int_flag)
+
                 min_y = work_df[target_now].min()
                 max_y = work_df[target_now].max()
                 work_df[target_now] = (work_df[target_now] - min_y) / (max_y - min_y)
@@ -1667,25 +1744,24 @@ class Snaplib:
 
                     test_y_all = np.array([])
                     pred_all = np.array([])
-                    
+
                     k_fold_dict = self.k_folds_split(work_df, target_now, K_FOLDS)
                     for k in range(0, K_FOLDS):
                         lgb_reg = lgb.LGBMRegressor(n_jobs=-1, random_state=0, device=device)
                         lgb_reg.fit(k_fold_dict['train_X'][k], k_fold_dict['train_y'][k])
                         pred = lgb_reg.predict(k_fold_dict['test_X'][k])
-                        if min_y >= 0.0:
-                            pred[pred < 0] = 0
+
                         test_y_all = np.concatenate(([test_y_all, k_fold_dict['test_y'][k]]), axis=None)
                         pred_all = np.concatenate((pred_all, pred), axis=None)
-                    
-                    if to_int_flag or len_target_values_counted <= CLASS_VALUE_COUNTS:
-                        pred_all = pred_all.astype(int).astype(float)
-                        pred_all[pred_all < min_y] = min_y
-                        pred_all[pred_all > max_y] = max_y
-                        
+
                     test_y_all = denormalize_targ(test_y_all, min_y, max_y)
                     pred_all = denormalize_targ(pred_all, min_y, max_y)
-
+                    
+                    if to_int_flag:
+                        pred_all = np.round(pred_all, 0)
+                    pred_all[pred_all < min_y] = min_y
+                    # pred_all[pred_all > max_y] = max_y
+                    
                     print(f'                       MAE: {mean_absolute_error(test_y_all, pred_all)}')
                     print(f'                      RMSE: {mean_squared_error(test_y_all,  pred_all) ** 0.5}')
 
@@ -1703,29 +1779,24 @@ class Snaplib:
 
                 X = work_df[predictors]
                 y = work_df[target_now]    
-                
+
                 # Final prediction & imputing
                 lgb_reg = lgb.LGBMRegressor(random_state=0, n_jobs=-1, device=device)
                 lgb_reg.fit(X, y)
                 pred_miss = lgb_reg.predict(miss_df)
                 pred_miss = denormalize_targ(pred_miss, min_y, max_y)
-                if min_y >= 0:
-                    pred_miss[pred_miss < 0] = 0
-                    
-                if to_int_flag or len_target_values_counted <= CLASS_VALUE_COUNTS:
-                    pred_miss = pred_miss.astype(int).astype(float)
-                    pred_miss[pred_miss < min_y] = min_y
-                    pred_miss[pred_miss > max_y] = max_y
-                    
+
+                if to_int_flag:
+                    pred_miss = np.round(pred_miss, 0)
+                pred_miss[pred_miss < min_y] = min_y
+                # pred_miss[pred_miss > max_y] = max_y
+
                 if verbose:
                     print(f'first 10 imputed: {list(np.round(pred_miss[:10], 1))}\n')
 
-
                 df.loc[miss_indeces, target_now] = np.array(pred_miss)
                 counter_predicted_values += len(miss_indeces)
 
-
-
             if verbose:
                 finish_iter_time = datetime.datetime.now()
                 requared = finish_iter_time - init_iter_time
@@ -1751,10 +1822,20 @@ class Snaplib:
             finish_time = datetime.datetime.now()
             requared = finish_time - init_time
             print(f'Required time totally: {str(requared)}\n\n')
-            
+
         unprocessed_list = list(data_info[data_info['NaN_counts'].notnull()].index)
         if unprocessed_list:
-            text = 'WARNING:\nUnprocessed columns :' + str(unprocessed_list) + '\nYou can try encoding or other methods'
+            text = 'WARNING:\nUnprocessed columns: ' + str(unprocessed_list) + '\nYou can try encoding or other methods'
             print(colored(text, 'red'))
 
-        return df
\ No newline at end of file
+        return df
+
+
+
+
+
+
+
+
+
+
